#!/bin/bash

PRINTER_NAME=zebra1.hq.ferrari-electronic.de
PRINTER_PORT=9100
TEMPLATE=template.zpl

DATE=$(date "+%m/%y")

MYNAME=$(basename "$0")

help()
{
    echo "usage: ${MYNAME} [-r <VARIABLE>=<VALUE>]  [-p <PRINTER>] [-P <PORT>] [-t <TEMPLATE>]"
    echo
    echo "options:"
    echo "  -r <VARIABLE>=<VALUE>  Replace variable in template by given value"
    echo "  -p <PRINTER>           Name of label printer"
    echo "  -P <PORT>              TCP port of label printer"
    echo "  -t <TEMPLATE>          Name of template ZPL file"
}

while [[ $# > 1 ]] ; do
    key="$1"
    
    case ${key} in
        -r)
            echo "$2"
            VAR_NAME=${2%=*}
            VAR_VALUE="${2#*=}"
            eval "${VAR_NAME}=\${VAR_VALUE}"
            shift
            ;;
        -p)
            PRINTER_NAME="$2"
            shift
            ;;
        -P)
            PRINTER_PORT="$2"
            shift
            ;;
        -t)
            TEMPLATE="$2"
            shift
            ;;
        -h|--help)
            help
            exit 0
            ;;
        *)
            # unknown option
            echo "unknown option ${key}"
            help
            exit 1
            ;;
    esac
    shift
done

if ! [ -r "${TEMPLATE}" ]; then
    echo "cannot open template ZPL file '${TEMPLATE}'"
    help
    exit 1
fi

# Scan template file for place holders and check if corresponding shell
# variables are not empty.
# Build a sed command to patch the template.
SED_ESCAPE_FIX_PARAMS="-e \"s/%V%G/@</g\" -e \"s/%I%V/>@/g\" -e \"s/[$]\+/@</g\" -e \"s/-[$]/>@/g\""
SED_PARAMS=""
for i in $(cat "${TEMPLATE}" | \
                  eval "sed ${SED_ESCAPE_FIX_PARAMS}" | \
                  grep -e "@<[^>]*>@" | \
                  sed -e "s/[^@]*@<\([^>]*\)>@[^@]*/\1 /g"); do
    if eval test -z \"\${${i}+x}\" ; then
        echo "template requires to patch a field '$i', but no appropriate value is given"
        help
        exit 1
    else
        SED_PARAMS="${SED_PARAMS} -e \"s;@<$i>@;$(eval echo \$$i);g\""
    fi
done

# patch the template and send it to printer port
cat "${TEMPLATE}" | \
    eval "sed ${SED_ESCAPE_FIX_PARAMS}" | \
    eval "sed ${SED_PARAMS}" | \
    sed "s/\^B3\([^,]*\),\([^,]*\),\([^,]*\),\([^,]*\),/^B3\1,Y,\3,\4,/g" | \
    nc ${PRINTER_NAME} ${PRINTER_PORT}
