require("json")

local omg={}

function omg.getHelpString(name,desc)
    local s=""
    s = s.. name .. " [options]\n"
    if desc ~= nil then
        s = s .. desc
    end
    s = s .. "\t--json\tgives output in json format\n"
    s = s .. "\t--xml\tgives output in XML format\n"
    s = s .. "\t--dot\tgives output in dotted ASCII format\n"
    s = s .. "\t--lua\tgives output in LUA format\n"
    s = s .. "\t--val\tvalues only for dotted output\n"
    s = s .. "\t--filter=[lua regexp]\t filters the output\n"

    return s
end

function omg.filterString(theString,filter)
    local s=""
    for line in string.gmatch(theString,"[^\n\r]+") do
        if string.find(line,filter) then
            s = s .. line .. "\n"
        end
    end    
    
    return s
end

function omg.tableToString(data, indent, maxdepth)
    local i
    local s = ""
    local out = ""

    if maxdepth == nil then
        maxdepth = 20
    end

    if indent == nil then
        i = 10
    else
        i = indent
    end

    s = string.rep(" ",i)

    for k,v in pairs(data) do
        if (type(v) == "table") then
            out = out .. s .. k .. "\n"
            if k ~= "_G" then
                if maxdepth > i then
                    out = out .. omg.tableToString(v,tonumber(tonumber(i)+3))
                else
                    out = out .. "MAXDEPTH REACHED"
                end
            end
        elseif (type(v) == "function") then
            --out = out .. sprintf("%s%s=%s\n",s,k,tostring(v))
        elseif (type(v) == "boolean") then
            out = out .. s .. k .. "=" .. tostring(v) .. "\n"
        elseif (type(v) == "userdata") then
            --out = out .. sprintf("%s%s=%s\n",s,k,tostring(v))
        else
            local n
            v,n = string.gsub(v,"\n","\n          ")
            if(n>0) then
                out = out .. s .. k .. "=\n" .. s .. v
            else
                out = out .. s .. k .. "=" .. v .. "\n"
            end
        end
    end

    return out
end

--This function dumps a table recursive
function omg.dumpTable(data, maxdepth)
    local s = omg.tableToString(data,0,maxdepth)
    print(s)
    io.flush ()
end

function omg.doTableToXml(data, indent, maxdepth)
    local i
    local s = ""
    local out = ""

    if maxdepth == nil then
        maxdepth = 20
    end

    if indent == nil then
        i = 10
    else
        i = indent
    end

    s = string.rep(" ",i)

    for k,v in pairs(data) do
        local key = k
        if tonumber(k) ~= nil then
            key = "id" .. k
        end 
        if (type(v) == "table") then
            out = out .. s .. "<" .. key .. ">\n"
            if k ~= "_G" then
                if maxdepth > i then
                    out = out .. omg.doTableToXml(v,tonumber(tonumber(i)+4))
                    out = out .. s .. "</" .. key .. ">\n"
                else
                    out = out .. "MAXDEPTH REACHED"
                end
            end
        elseif (type(v) == "function") then
            --out = out .. sprintf("%s%s=%s\n",s,k,tostring(v))
        elseif (type(v) == "boolean") then
            out = out .. s .. "<" .. key .. ">" .. tostring(v) .. "</" .. key .. ">\n"
        elseif (type(v) == "userdata") then
            --out = out .. sprintf("%s%s=%s\n",s,k,tostring(v))
        else
            v = string.gsub(v, '<', '&lt;')
            v = string.gsub(v, '>', '&gt;')
            out = out .. s .. "<" .. key .. ">" .. v .. "</" .. key .. ">\n"
        end
    end

    return out
end

function omg.tableToXml(data, firstTag, maxdepth)
    local s = '<?xml version="1.0"?>\n'
    s = s .. "<" .. (firstTag or "NIL") .. ">\n"
    s = s .. omg.doTableToXml(data,4,maxdepth)
    s = s .. "</" .. (firstTag or "NIL") .. ">\n"
    return s
end

function omg.tableToJson(t)
    local jsonString=json.encode(t)
    return jsonString
end

function omg.captureN(cmd)
    local f = assert(io.popen(cmd, 'r'))
    local s = assert(f:read('*a'))
    f:close()
    s = string.gsub(s, '^%s+', '')
    s = string.gsub(s, '%s+$', '')
    s = string.gsub(s, '[\n\r]+', ' ')
    return s
end

function omg.capture(cmd)
    local f = assert(io.popen(cmd, 'r'))
    local s = assert(f:read('*a'))
    f:close()
    return s
end

function omg.cmd2TreeTable(command,delimiter,assignChar)
    local returnTable={}
    local mySplit={}
    local line

    local t = returnTable

    stdout=omg.capture("LC_ALL=C "..command.." 2>&1")
    for line in stdout:gmatch("[^\n\r]+")do
        if(line ~= nil) then
            local _,_,kEnd = string.find(line,"([^" .. delimiter .. "]-)" .. assignChar)
            for k, v in string.gmatch(line, "([^" .. assignChar .. "]+)" .. assignChar .. "(.+)") do
                --print(k,v,kEnd)
                for v2 in string.gmatch(k, "[^" .. delimiter .. "]+") do
                    if v2 ~= kEnd then
                        --print(v2)
                        if t[v2] == nil then
                            t[v2]={}
                        end
                        t=t[v2]
                    end
                end
                local len = string.len(v)
                --print(t,k,v, string.len(v), string.byte(v,len,len))
                if string.byte(v,len,len) == 0 then
                    v = string.sub(v,1,len-1)
                end
                v = string.gsub(v,'"','')
                t[kEnd]=string.gsub(v,"\0","")
                t=returnTable
            end
        end
    end
    return returnTable
end

function omg.string2TreeTable(theString,delimiter,assignChar)
    local returnTable={}
    local mySplit={}
    local line

    local t = returnTable

    for line in string.gmatch(theString,"[^\n\r]+")do
        if(line ~= nil) then
            local _,_,kEnd = string.find(line,"([^" .. delimiter .. "]-)" .. assignChar)
            for k, v in string.gmatch(line, "([^" .. assignChar .. "]+)" .. assignChar .. "(.+)") do
                --print(k,v,kEnd)
                for v2 in string.gmatch(k, "[^" .. delimiter .. "]+") do
                    if v2 ~= kEnd then
                        --print(v2)
                        if t[v2] == nil then
                            t[v2]={}
                        end
                        t=t[v2]
                    end
                end
                local len = string.len(v)
                --print(t,k,v, string.len(v), string.byte(v,len,len))
                if string.byte(v,len,len) == 0 then
                    v = string.sub(v,1,len-1)
                end
                v = string.gsub(v,'"','')
                t[kEnd]=string.gsub(v,"\0","")
                t=returnTable
            end
        end
    end
    return returnTable
end

function omg.cmd2LineTreeTable(command,delimiter,assignChar)
    local returnTable={}
    local mySplit={}
    local line
    local lineCount=1

    local t = returnTable
    stdout=omg.capture("LC_ALL=C "..command.." 2>&1")
    for line in stdout:gmatch("[^\n\r]+")do
        if(line ~= nil) then

	    t[lineCount]={}
    	    t=t[lineCount]    

            local _,_,kEnd = string.find(line,"([^" .. delimiter .. "]-)" .. assignChar)
            for k, v in string.gmatch(line, "([^" .. assignChar .. "]+)" .. assignChar .. "(.+)") do
                --print(k,v,kEnd)
                for v2 in string.gmatch(k, "[^" .. delimiter .. "]+") do
                    if v2 ~= kEnd then
                        --print(v2)
                        if t[v2] == nil then
                            t[v2]={}
                        end
                        t=t[v2]
                    end
                end
                local len = string.len(v)
                --print(t,k,v, string.len(v), string.byte(v,len,len))
                if string.byte(v,len,len) == 0 then
                    v = string.sub(v,1,len-1)
                end
                v = string.gsub(v,'"','')
                t[kEnd]=string.gsub(v,"\0","")
                t=returnTable
            end

	    lineCount = lineCount + 1
        end
    end
    return returnTable
end

function omg.tableToDelimiterList(data, delimiter, assignchar, prefix, valonly)
    local myPrefix
    local out = ""

    if prefix == nil then
       prefix=""
    end

    for k,v in pairs(data) do
        if (type(v) == "table") then
            if valonly==true then
                myPrefix = prefix
            else
                myPrefix = prefix .. k .. delimiter
            end
            out = out .. omg.tableToDelimiterList(v, delimiter, assignchar, myPrefix, valonly)
        elseif (type(v) == "function") then
            --out = out .. 
        elseif (type(v) == "userdata") then
            --out = out .. 
        elseif (type(v) == "boolean") then
            if valonly==true then
                out = out .. prefix .. tostring(v) .. "\n"
            else
                out = out .. prefix .. k .. assignchar .. tostring(v) .. "\n"
            end
        else
            if valonly==true then
                out = out .. prefix .. v .. "\n"
            else
               if string.find(v," ") then
                  out = out .. prefix .. k .. assignchar .. "\"" .. v .. "\"\n"
               else
                  out = out .. prefix .. k .. assignchar .. v .. "\n"
               end
            end
        end
    end

    return out
end

function omg.dumpTableToDelimiterList(data, delimiter, assignchar, valonly, filter, prefix)
    local s = omg.tableToDelimiterList(data, delimiter, assignchar, prefix, valonly)
    if filter ~= nil then
        s=omg.filterString(s,filter)
    end
    return s
end

function filterTable(theTable,filter)
    local s = omg.dumpTableToDelimiterList(theTable,".","=",nil,filter)
    local t = omg.string2TreeTable(s, ".", "=")
    return t
end

function omg.makeTablePath(theTable,path)
    if theTable==nil then
        theTable={}
    end

    local t = theTable

    for v in string.gmatch(path, "([^.]+)") do
        if t[v] == nil then
            t[v]={}
        end
        t=t[v]
    end

    return theTable
end

function omg.splitCommandlineString(arguments)
    local jcargs = {}
    local argumentFollow=false
    local savedName
    local k,v
    
    if log then print("start splitCommandlineString") end
    for k,v in ipairs(arguments) do
        if log then print(k,v) end
        if argumentFollow then
            if string.find(v,"^%-") then
                if savedName then
                    jcargs[savedName]="true"
                    if log then print("1) set ",savedName,"=","true") end
                    savedName=nil
                end
                _,_,savedName=string.find(v,"^.(.*)")
                if log then  print("debug 2 -->",v,savedName) end
                argumentFollow=true
            else
                if savedName then
                    jcargs[savedName]=v
                    if log then print("2) set ",savedName,"=",v) end
                    savedName=nil
                end
                argumentFollow=false
            end
        elseif string.find(v,"%-%-.-=") then
            local name
            local value
            _,_,name,value=string.find(v,"%-%-(.-)=(.*)")
            jcargs[name]=value or ""
            if log then print("3) set ",name,"=",value) end
        elseif string.find(v,"%-%-") then
            local name
            _,_,name=string.find(v,"%-%-(.*)")
            jcargs[name]=true
            if log then print("4) set ",name,"=","true") end
        elseif string.find(v,"=") then
            local name
            local value
            _,_,name,value=string.find(v,"(.-)=(.*)")
            jcargs[name]=value or ""
            if log then print("3) set ",name,"=",value) end
        elseif string.find(v,"^%-") then
            _,_,savedName=string.find(v,"^.(.*)")
            if log then print("debug 1 -->",v,savedName) end
            argumentFollow=true
        else
            jcargs[v]=v
            if log then print("5) set ",v,"=",v) end
        end
    end

    if argumentFollow then
        if savedName then
            jcargs[savedName]="true"
            if log then print("6) set ",savedName,"=","true") end
            savedName=nil
        end
    end

    if log then 
        print("params:")
        for k,v in pairs(jcargs) do
            print(k,v)
        end
    end

    if log then print("stop SplitCommandlineString") end
    return jcargs
end

--[[ This function is splits message in table
--]]
function omg.messageToTable(data)
    --assert(loadstring(data))()
    --return _m

    local f,text

    _m = nil
    
    f,text = loadstring(data)
    if f == nil then
        debug(0,"can't interpret '%s'",data or "NIL")
        return nil
    else
        f()
        return _m
    end
end

--[[ This function is generate table to message (only one level)
--]]
function omg.tableToMessage(data,s,i)
    local key
    if s==nil then s="_m = {}\n" end
    if i==nil then i="_m[" end

    for k,v in pairs(data) do
        if  type(k)=="number" then
            key=k
        else
            key = "\"" .. k .. "\""
        end

        if (type(v) == "table") then
            s = s .. i .. key .. "]={}\n"
            s = omg.tableToMessage(v,s,i .. key .. "][")
        elseif (type(v) == "string") then
            local v1,_ = string.gsub(v,"\"","\\\"")
            local v2,_ = string.gsub(v1,"\r\n","\\r\\n\"..\r\n\"")
            s = s .. i .. key .. "]=\"" .. v2 .. "\"\n"
        elseif (type(v) == "boolean") then
            s = s .. i .. key .. "]=" .. tostring(v) .. "\n"
        else
            s = s .. i .. key .. "]=" .. v .. "\n"
        end
    end

    return s
end

-- see if the file exists
function omg.fileExists(file)
    local f = io.open(file, "rb")
    if f then f:close() end
    return f ~= nil
end

function omg.execute(cmd, raw, pos)
    local l=1

    local f = assert(io.popen(cmd, 'r'))
    local s = assert(f:read('*a'))
    f:close()

    if pos~=nil and raw then
        s,l = string.gsub(s,  "\n", '\n' .. string.rep(" ",pos-1))
    end

    if raw then 
        return s,l 
    end

    s = string.gsub(s, '^%s+', '')
    s = string.gsub(s, '%s+$', '')
    s = string.gsub(s, '[\n\r]+', ' ')
    return s,l
end

function omg.calcIpv4Bitmask(bits)
    local i = tonumber(bits)
    m = {}
    k = 4;

    while i >= 8 do
        m[k]=255
        k=k-1
        i=i-8
    end

    if i == 1 then
        m[k]=128
    elseif i==2 then
        m[k]=192
    elseif i==3 then
        m[k]=224
    elseif i==4 then
        m[k]=240
    elseif i==5 then
        m[k]=248
    elseif i==6 then
        m[k]=252
    elseif i==7 then
        m[k]=254
    end
    if i>0 then
        k=k-1
    end

    while k > 0 do
        m[k]=0
        k=k-1
    end

    local mask = string.format("%d.%d.%d.%d",m[4],m[3],m[2],m[1])
    return mask
end

function omg.calcIpv6Bitmask(bits)
    local i = tonumber(bits)
    m = {}
    k = 16;

    while i >= 8 do
        m[k]=255
        k=k-1
        i=i-8
    end

    if i == 1 then
        m[k]=128
    elseif i==2 then
        m[k]=192
    elseif i==3 then
        m[k]=224
    elseif i==4 then
        m[k]=240
    elseif i==5 then
        m[k]=248
    elseif i==6 then
        m[k]=252
    elseif i==7 then
        m[k]=254
    end
    if i>0 then
        k=k-1
    end

    while k > 0 do
        m[k]=0
        k=k-1
    end

    local mask = string.format("%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x",m[16],m[15],m[14],m[13],m[12],m[11],m[10],m[9],m[8],m[7],m[6],m[5],m[4],m[3],m[2],m[1])

    return mask
end

function omg.ipConfig()
    local mode="interface"
    local t={}
    local currentInterface
    local ip4count
    local ip6count

    local c=omg.execute('/sbin/ip addr', true)
    for line in string.gmatch(c,"[^\n\r]+") do
        local startpos,endpos,key,interface,mtu,state=string.find(line,"^(%d-): (%w-): .* mtu ([^ ]*) .* state ([^ ]*)")
        --print("1. ",startpos,endpos,key,interface,mtu,state," ",line)

        if startpos==nil then
            startpos,endpos,key,interface,mtu=string.find(line,"^(%d-): (%w-): .* mtu ([^ ]*)")
            --print("2. ",startpos,endpos,key,interface,mtu,state," ",line)
        end

        if  startpos~=nil then
            mode="interface"
        else
            mode="properties"
        end

        if mode == "interface" then
            currentInterface=interface
            ip4count=1
            ip6count=1
            t[interface]={}
            t[interface].available=true
            t[interface].mtu=mtu
            if state ~= nil then
                t[interface].state=string.lower(state)
            end
        elseif mode == "properties" then
            local found=false

            --local
            if not found and currentInterface~=nil then
                local startpos,endpos,prop,address,mask,scope=string.find(line,"^%s-(inet) ([^ /]*)/([^ ]*) scope ([^ ]*)")
                --print(startpos,endpos,prop,address,mask,scope," ",line)
                if startpos ~= nil then
                    if t[currentInterface][prop] == nil then
                        t[currentInterface][prop]={}
                    end
                    t[currentInterface][prop][1]={}
                    t[currentInterface][prop][1].isIpV4=true
                    t[currentInterface][prop][1].address=address
                    t[currentInterface][prop][1].mask=mask
                    t[currentInterface][prop][1].ipv4mask=omg.calcIpv4Bitmask(mask)
                    t[currentInterface][prop][1].scope=scope
                    found=true
                end
            end            

            --ipv4
            if not found and currentInterface~=nil then
                local startpos,endpos,prop,address,mask,broadcast,scope=string.find(line,"^%s-(inet) ([^ /]*)/([^ ]*) brd ([^ ]*) scope ([^ ]*)")
                --print(startpos,endpos,prop,address,mask,broadcast,scope," ",line)
                if startpos ~= nil then
                    if t[currentInterface][prop] == nil then
                        t[currentInterface][prop]={}
                    end
                    t[currentInterface][prop][ip4count]={}
                    t[currentInterface][prop][ip4count].address=address
                    t[currentInterface][prop][ip4count].masklength=mask
                    t[currentInterface][prop][ip4count].mask=omg.calcIpv4Bitmask(mask)
                    t[currentInterface][prop][ip4count].broadcast=broadcast
                    t[currentInterface][prop][ip4count].scope=scope
                    found=true
                    ip4count=ip4count+1
                end
            end            

            --ipv6
            if not found and currentInterface~=nil then
                local startpos,endpos,prop,address,mask,scope=string.find(line,"^%s-(inet6) ([^ /]*)/([^ ]*) scope ([^ ]*)")
                --print(startpos,endpos,prop,address,mask,scope," ",line)
                if startpos ~= nil then
                    if t[currentInterface][prop] == nil then
                        t[currentInterface][prop]={}
                    end
                    t[currentInterface][prop][ip6count]={}
                    t[currentInterface][prop][ip6count].address=address
                    t[currentInterface][prop][ip6count].masklength=mask
                    t[currentInterface][prop][ip6count].mask=omg.calcIpv6Bitmask(mask)
                    t[currentInterface][prop][ip6count].scope=scope
                    found=true
                    ip6count=ip6count+1
                end
            end            

            if not found and currentInterface~=nil then
                local startpos,endpos,prop,type,macAddress=string.find(line,"^%s-(link)/([^ ]*) ([^ ]*) brd ([^ ]*)")
                --print(startpos,endpos,prop,address,mask,scope," ",line)
                if startpos ~= nil then
                    t[currentInterface][prop]={}
                    t[currentInterface][prop].type=type
                    t[currentInterface][prop].macAddress=macAddress
                    found=true
                end
            end            
        end
    end    
    return t;
end

function omg.getResolvConf()
    local val
    val=omg.cmd2LineTreeTable("cat /etc/resolv.conf 2>/dev/null | grep -v '#'", ".", " ")
    return val
end

function omg.getOsType()
    local val=string.lower(omg.execute('if [ -e /etc/redhat-release ]; then cat /etc/redhat-release | head -n 1 | cut -d " " -f 1 ;else if [ -r /etc/issue ] ;then cat /etc/issue | head -n 1 | cut -d " " -f 1 ;fi ;fi') or "unknown")
    return val
end

function omg.getOsVersion(os)
    local val="unknown"
    
    if os == nil then
        os=omg.getOsType()
    end

    if os == "ubuntu" then
        val=omg.execute('cat /etc/issue | head -n 1 | cut -d " " -f 2')
    elseif os == "centos" then
        val=omg.execute('cat /etc/redhat-release | sed -e "s/[^0-9.]//g" | cut -d "." -f 1')
    elseif os == "red" then
        val=omg.execute('cat /etc/redhat-release | sed -e "s/[^0-9.]//g" | cut -d "." -f 1')
    end
    return val
end

function omg.getOsVersionName(os)
    local val="unknown"
    
    if os == nil then
        os=omg.getOsType()
    end

    if os == "ubuntu" then
        val=omg.execute('lsb_release -c | cut  -f 2')
    elseif os == "centos" then
        val=omg.execute('cat /etc/redhat-release | sed -e "s/.* \\(.*\\)/\\1/" | sed -e "s/(//" -e "s/)//"')
    elseif os == "red" then
        val=omg.execute('cat /etc/redhat-release | sed -e "s/.* \\(.*\\)/\\1/" | sed -e "s/(//" -e "s/)//"')
    end
    return val
end

function omg.getPackages(os)
    local val

    if os == nil then
        os=omg.getOsType()
    end

    if os == "ubuntu" then
        --val=omg.cmd2TreeTable("dpkg-query -W -f='${Package}=${Version}\n'|grep omg-", ".", "=")
	val=omg.cmd2TreeTable("dpkg-query  -f='${Package}.version=${Version}\n${Package}.summary=\"${Description}\"\n' -W 'omg-*'",".","=")
    elseif os == "centos" then
        val=omg.cmd2TreeTable('rpm -q --queryformat "%{NAME}.version=%{VERSION}-%{RELEASE}\n%{NAME}.summary=\"%{SUMMARY}\"\n" omg omg2 omg-modules', ".", "=")

    elseif os == "red" then
        val=omg.cmd2TreeTable('rpm -q --queryformat "%{NAME}.version=%{VERSION}-%{RELEASE}\n%{NAME}.summary=\"%{SUMMARY}\"\n" omg omg2 omg-modules', ".", "=")
    end

    return val
end

function omg.getCmdline()
    local val
    val=omg.cmd2TreeTable("cat /tmp/cmdline 2>/dev/null | grep -v '#' | sed -e 's/^export //'", ".", "=")
    return val
end

function omg.getHwVendor()
    local val
    val=omg.execute('dmidecode -s system-manufacturer')
    return val
end

function omg.getHwProduct()
    local val
    val=omg.execute('dmidecode -s system-product-name')
    return val
end

function omg.getHwProcessorVersion()
    local val
    val=omg.execute('dmidecode -s processor-version')
    return val
end

function omg.getHwProcessorFrequency()
    local val
    val=omg.execute('dmidecode -s processor-frequency')
    return val
end

function omg.getHwProcessorFamily()
    local val
    val=omg.execute('dmidecode -s processor-family')
    return val
end

function omg.getHwProcessorManufacturer()
    local val
    val=omg.execute('dmidecode -s processor-manufacturer')
    return val
end

function omg.getSystemUuid()
    local val
    val=omg.execute('dmidecode -s system-uuid')
    return val
end

function omg.getFeatureversion()
    local val
    val=omg.execute('featureversion')
    return val
end

function omg.runScript(theTable,arg,description)
    log=false

    options=omg.splitCommandlineString(arg)

    if options.filter~=nil then
        theTable = filterTable(theTable,options.filter)
    end

    if options.help then
        io.write(omg.getHelpString(arg[0],description .. "\n"))
    elseif options.json then
        io.write(omg.tableToJson(theTable) .. "\n")
    elseif options.xml then
        io.write(omg.tableToXml(theTable,"omgsysteminfo"))
    elseif options.dot then
        io.write(omg.dumpTableToDelimiterList(theTable,".","=",options.val))
    elseif options.lua then
        io.write(omg.tableToMessage(theTable))
    else
        io.write(omg.dumpTableToDelimiterList(theTable,".","=",options.val))
    end
end

return omg
